"""
Utils init script
From scrnatools package

Created on Mon Jan 10 15:57:46 2022

@author: joe germino (joe.germino@ucsf.edu)
"""

# import functions from their modules

from ._type_check import type_check
from ._debug import debug
from ._fill_array import fill_array
from ._check_path import check_path
